__title__ = 'ofxstatement plugin for dutch financial institutions'
__package_name__ = 'ofxstatement-dutch'
__author__ = "Gert-Jan Paulissen"
__description__ = ' '.join('''
OFXStatement plugin for dutch financial institutions like DEGIRO, ICSCards,
ING, KNAB and ASN.
'''.strip().split())
__email__ = "gert.jan.paulissen@gmail.com"
__version__ = '1.7.0'
__license__ = "GPLv3"
__copyright__ = 'Copyright 2020-2024 Gert-Jan Paulissen'
__url__ = "https://github.com/gpaulissen/ofxstatement-dutch"

if __name__ == '__main__':
    print(__version__)
